{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import torch\n",
    "print(torch.__version__)"
   ],
   "id": "0194d86c-7f59-47f8-9dfa-c5d367aa8b42"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import torchvision\n",
    "print(torchvision.__version__)"
   ],
   "id": "80954cec-d00e-4aa0-ba91-924380b638be"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import os, time\n",
    "\n",
    "import numpy as np\n",
    "import matplotlib.pylab as plt\n",
    "\n",
    "import torch\n",
    "import torch.nn as nn\n",
    "import torch.nn.functional as F\n",
    "from torch import optim\n",
    "import torchvision\n",
    "from torchvision.transforms import v2\n",
    "from torchvision import datasets\n",
    "\n",
    "plt.rcParams['figure.figsize'] = (10, 6)"
   ],
   "id": "6c6302f3-6689-4318-8a78-878352d9e2cf"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "torch.cuda.is_available()"
   ],
   "id": "afb4ab40-7382-4924-801b-a6187791415e"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Classification MNIST-Fashion\n",
    "\n",
    "De nombreux datasets sont accessibles directement: voir\n",
    "<https://pytorch.org/docs/stable/torchvision/datasets.html>\n",
    "\n",
    "PyTorch utilise une classe DataLoader qui permet de gérer l’accès aux\n",
    "données (en particulier, en gérant efficamenet le parallélisme et les\n",
    "batchs): voir\n",
    "<https://pytorch.org/docs/stable/data.html#torch.utils.data.DataLoader>\n",
    "\n",
    "De plus, de nombreuses tranformations sont disponbiles pour réaliser\n",
    "l’augmentation des données: voir\n",
    "<https://pytorch.org/docs/stable/torchvision/transforms.html>\n",
    "\n",
    "On charge ici un dataset prédéfini\n",
    "<https://pytorch.org/docs/stable/torchvision/datasets.html#torchvision.datasets.FashionMNIST>\n",
    "\n",
    "Il s’agit de la base d’images décrite ici:\n",
    "<https://github.com/zalandoresearch/fashion-mnist>"
   ],
   "id": "a5401c64-0a02-4bbd-a64b-3b54e78ec108"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "train_loader = torch.utils.data.DataLoader(\n",
    "    torchvision.datasets.FashionMNIST('fashionmnist', train=True, download=True,\n",
    "                   transform=torchvision.transforms.Compose([\n",
    "                       torchvision.transforms.ToTensor(),\n",
    "                       torchvision.transforms.Normalize((0.1307,), (0.3081,))\n",
    "                       ])),\n",
    "    batch_size=64, shuffle=True)\n",
    "test_loader = torch.utils.data.DataLoader(\n",
    "    torchvision.datasets.FashionMNIST('fashionmnist', train=False,\n",
    "                               transform=torchvision.transforms.Compose([\n",
    "                                   torchvision.transforms.ToTensor(),\n",
    "                                   torchvision.transforms.Normalize((0.1307,), (0.3081,))\n",
    "                                   ])),\n",
    "    batch_size=1000, shuffle=True)"
   ],
   "id": "513215d7-d1af-4a71-98ab-a7d3ae673290"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class Net(nn.Module):\n",
    "    def __init__(self):\n",
    "        super(Net, self).__init__()\n",
    "        self.conv1 = nn.Conv2d(1, 10, kernel_size=5)\n",
    "        self.conv2 = nn.Conv2d(10, 20, kernel_size=5)\n",
    "        self.conv2_drop = nn.Dropout2d()\n",
    "        self.fc1 = nn.Linear(320, 50)\n",
    "        self.fc2 = nn.Linear(50, 10)\n",
    "\n",
    "    def forward(self, x):\n",
    "        x = F.relu(F.max_pool2d(self.conv1(x), 2))\n",
    "        x = F.relu(F.max_pool2d(self.conv2_drop(self.conv2(x)), 2))\n",
    "        x = x.view(-1, 320)\n",
    "        x = F.relu(self.fc1(x))\n",
    "        x = F.dropout(x, training=self.training)\n",
    "        x = self.fc2(x)\n",
    "\n",
    "        return F.log_softmax(x, dim=1)"
   ],
   "id": "cf6e9298-921b-40c0-8bae-76ce9a6ac70b"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "model = Net()\n",
    "\n",
    "device = torch.device(\"cuda:0\" if torch.cuda.is_available() else \"cpu\")\n",
    "model = model.to(device)"
   ],
   "id": "43ec72cd-46ba-4755-a912-c66028161a4a"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Optimizer :\n",
    "<https://pytorch.org/docs/stable/optim.html#module-torch.optim>"
   ],
   "id": "e3de2405-6e41-460a-bb0b-10901f39e749"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "optimizer = optim.SGD(model.parameters(), lr=0.01)"
   ],
   "id": "4d52013a-5a4c-475d-b699-dc66e97fb105"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Fonction de loss\n",
    "<https://pytorch.org/docs/stable/nn.html#loss-functions>"
   ],
   "id": "83b0a1de-0c30-4e3b-89dc-838622c4a4d5"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "criterion = torch.nn.NLLLoss()"
   ],
   "id": "6553b942-c7ab-4ef0-bc43-3cbeb34be472"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Fonction de train:"
   ],
   "id": "9a44e10a-e7b3-453a-9f91-18fe59131a2d"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def train(model, device, train_loader, optimizer, epoch, criterion):\n",
    "    model.train()\n",
    "    for batch_idx, (data, target) in enumerate(train_loader):\n",
    "        data, target = data.to(device), target.to(device)\n",
    "        optimizer.zero_grad()\n",
    "\n",
    "        output = model(data)\n",
    "        loss = criterion(output, target)\n",
    "        loss.backward()\n",
    "        optimizer.step()\n",
    "        \n",
    "        if batch_idx % 100 == 0:\n",
    "            print('Train Epoch: {} [{}/{} ({:.0f}%)]\\tLoss: {:.6f}'.format(\n",
    "                epoch, batch_idx * len(data), len(train_loader.dataset),\n",
    "                100. * batch_idx / len(train_loader), loss.item()))"
   ],
   "id": "75dc7261-feeb-4752-a0d2-348f2a2900d3"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Fonction de test:"
   ],
   "id": "44465959-33d5-463c-8bbc-5419caeb196f"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def test(model, device, test_loader, criterion):\n",
    "    model.eval()\n",
    "    test_loss = 0\n",
    "    correct = 0\n",
    "    with torch.no_grad():\n",
    "        for data, target in test_loader:\n",
    "            data, target = data.to(device), target.to(device)\n",
    "            output = model(data)\n",
    "            test_loss += criterion(output, target).item() # sum up batch loss\n",
    "            pred = output.max(1, keepdim=True)[1] # get the index of the max log-probability\n",
    "            correct += pred.eq(target.view_as(pred)).sum().item()\n",
    "\n",
    "    test_loss /= len(test_loader.dataset)\n",
    "    print('\\nTest set: Average loss: {:.4f}, Accuracy: {}/{} ({:.0f}%)\\n'.format(\n",
    "        test_loss, correct, len(test_loader.dataset),\n",
    "        100. * correct / len(test_loader.dataset)))\n",
    "    return "
   ],
   "id": "7777becc-869f-4261-b0ce-dcd420a76a1f"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "for epoch in range(1, 10):\n",
    "        start_time = time.process_time()\n",
    "        train(model, device, train_loader, optimizer, epoch,criterion)\n",
    "        end_time = time.process_time()\n",
    "        print(\"Train Epoch duration {:.2f}s\".format(end_time - start_time))\n",
    "        \n",
    "        test(model, device, test_loader,criterion)"
   ],
   "id": "8d15e09f-25b3-4f0c-bdf5-34066758ae32"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "    Train Epoch: 1 [0/60000 (0%)]   Loss: 2.331967\n",
    "    Train Epoch: 1 [6400/60000 (11%)]   Loss: 2.220635\n",
    "    Train Epoch: 1 [12800/60000 (21%)]  Loss: 2.072088\n",
    "    Train Epoch: 1 [19200/60000 (32%)]  Loss: 1.749465\n",
    "    Train Epoch: 1 [25600/60000 (43%)]  Loss: 1.258822\n",
    "    Train Epoch: 1 [32000/60000 (53%)]  Loss: 1.271635\n",
    "    Train Epoch: 1 [38400/60000 (64%)]  Loss: 1.171696\n",
    "    Train Epoch: 1 [44800/60000 (75%)]  Loss: 1.161705\n",
    "    Train Epoch: 1 [51200/60000 (85%)]  Loss: 1.034610\n",
    "    Train Epoch: 1 [57600/60000 (96%)]  Loss: 0.904365\n",
    "    Train Epoch duration 95.98s\n",
    "\n",
    "    Test set: Average loss: 0.0008, Accuracy: 6984/10000 (70%)\n",
    "\n",
    "    [...]\n",
    "\n",
    "    Train Epoch: 9 [0/60000 (0%)]   Loss: 0.813664\n",
    "    Train Epoch: 9 [6400/60000 (11%)]   Loss: 0.643290\n",
    "    Train Epoch: 9 [12800/60000 (21%)]  Loss: 0.792572\n",
    "    Train Epoch: 9 [19200/60000 (32%)]  Loss: 0.680120\n",
    "    Train Epoch: 9 [25600/60000 (43%)]  Loss: 0.951693\n",
    "    Train Epoch: 9 [32000/60000 (53%)]  Loss: 0.531021\n",
    "    Train Epoch: 9 [38400/60000 (64%)]  Loss: 0.510627\n",
    "    Train Epoch: 9 [44800/60000 (75%)]  Loss: 0.764454\n",
    "    Train Epoch: 9 [51200/60000 (85%)]  Loss: 0.707269\n",
    "    Train Epoch: 9 [57600/60000 (96%)]  Loss: 0.570231\n",
    "    Train Epoch duration 132.90s\n",
    "\n",
    "    Test set: Average loss: 0.0005, Accuracy: 8193/10000 (82%)\n",
    "\n",
    "## À ajouter\n",
    "\n",
    "Plot en fonction des epoch:\n",
    "\n",
    "-   training loss\n",
    "-   testing loss\n",
    "-   accuracy"
   ],
   "id": "4e04ae11-7238-478c-8ce7-9e4ba192e5a7"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [],
   "id": "a54004d9-4163-44b8-9143-fd028304bd1a"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Fine-tuning\n",
    "\n",
    "Les données sont disponibles à l’adresse\n",
    "<https://download.pytorch.org/tutorial/hymenoptera_data.zip>\n",
    "\n",
    "## Téléchargement des données"
   ],
   "id": "67414348-085f-4021-9ee6-d2d64d3c795c"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "!wget https://download.pytorch.org/tutorial/hymenoptera_data.zip\n",
    "!unzip hymenoptera_data.zip"
   ],
   "id": "17a9afab-229a-4e08-a030-848529a9799b"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Augmentation de données basique et très classique"
   ],
   "id": "f5102f42-be7c-4ac8-b46e-6355e5673e6b"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "data_transforms = {\n",
    "    # Data augmentation and normalization for training\n",
    "    'train': v2.Compose([\n",
    "        v2.RandomResizedCrop(224),\n",
    "        v2.RandomHorizontalFlip(),\n",
    "        v2.ToImage(),\n",
    "        v2.ToDtype(torch.float32, scale=True),\n",
    "        v2.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])\n",
    "    ]),\n",
    "    # Just normalization for validation\n",
    "    'val': v2.Compose([\n",
    "        v2.Resize(256),\n",
    "        v2.CenterCrop(224),\n",
    "        v2.ToImage(),\n",
    "        v2.ToDtype(torch.float32, scale=True),\n",
    "        v2.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])\n",
    "    ]),\n",
    "}"
   ],
   "id": "62b89638-0f5d-461a-887c-7ee8ce2fef11"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Construction d’un dataloader\n",
    "\n",
    "Voir <https://pytorch.org/vision/stable/datasets.html> pour les datasets\n",
    "classiques et aussi\n",
    "<https://pytorch.org/vision/stable/generated/torchvision.datasets.DatasetFolder.html#torchvision.datasets.DatasetFolder>\n",
    "pour faire des datasets personnalisés."
   ],
   "id": "03ac1560-cb2f-4f35-a4a5-116e7744934d"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "data_dir = 'hymenoptera_data'\n",
    "image_datasets = {x: datasets.ImageFolder(os.path.join(data_dir, x),\n",
    "                                          data_transforms[x])\n",
    "                  for x in ['train', 'val']}\n",
    "dataloaders = {x: torch.utils.data.DataLoader(image_datasets[x], batch_size=4,\n",
    "                                             shuffle=True, num_workers=4)\n",
    "              for x in ['train', 'val']}\n",
    "dataset_sizes = {x: len(image_datasets[x]) for x in ['train', 'val']}\n",
    "class_names = image_datasets['train'].classes\n",
    "\n",
    "device = torch.device(\"cuda:0\" if torch.cuda.is_available() else \"cpu\")"
   ],
   "id": "1b0f5783-c847-456a-9e08-b65e5120ea3f"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def imshow(inp, title=None):\n",
    "    \"\"\"Imshow for Tensor.\"\"\"\n",
    "    inp = inp.numpy().transpose((1, 2, 0))\n",
    "    mean = np.array([0.485, 0.456, 0.406])\n",
    "    std = np.array([0.229, 0.224, 0.225])\n",
    "    inp = std * inp + mean\n",
    "    inp = np.clip(inp, 0, 1)\n",
    "    plt.imshow(inp)\n",
    "    if title is not None:\n",
    "        plt.title(title)\n",
    "    plt.pause(0.001)  # pause a bit so that plots are updated\n",
    "\n",
    "\n",
    "# Get a batch of training data\n",
    "inputs, classes = next(iter(dataloaders['train']))\n",
    "\n",
    "# Make a grid from batch\n",
    "out = torchvision.utils.make_grid(inputs)\n",
    "\n",
    "imshow(out, title=[class_names[x] for x in classes])"
   ],
   "id": "92234d46-be15-4c80-9019-2087c0a8d240"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Resnet\n",
    "\n",
    "Voir\n",
    "<https://github.com/pytorch/vision/blob/master/torchvision/models/resnet.py>\n",
    "pour comprendre comment couper le réseau."
   ],
   "id": "c159e635-a70c-45b4-a84f-799f85c7848d"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "from torchvision.models import resnet18\n",
    "\n",
    "model = resnet18(weights='IMAGENET1K_V1') # Modèle pré-entraîné sur ImageNet"
   ],
   "id": "2264a51e-d353-4de0-93d1-cf1abed239e5"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "print(model)"
   ],
   "id": "ffdbfb05-c17d-4818-8b8a-6d6fe786d78a"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "On fixe les paramètres de toutes les couches, ils ne seront pas modifiés\n",
    "lors de l’apprentissage."
   ],
   "id": "a11e07e5-0238-4517-bb65-5e9976a3d6c0"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "for param in model.parameters():\n",
    "    param.requires_grad = False"
   ],
   "id": "cddfa997-30c8-4592-b68d-44648bb7671d"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "On remplace la partie finale par une simple couche dense (qui sera elle\n",
    "apprise)."
   ],
   "id": "abeaca78-d1fc-446d-8fc3-b8ffac069719"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "model.fc = nn.Linear(512, 2)"
   ],
   "id": "532042d6-864b-4f00-9619-9fcda575656e"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Calcul de la représentation, pour l'envoyer à un SVM par exemple\n",
    "# model.avgpool(x) # sortie de dernièer couche avant le classifier\n",
    "# Ce serait un prétraitement à appliquer à toutes les données avant d'entraîner le SVM"
   ],
   "id": "8b8136c1-a338-41b8-b0f9-486bccf0967b"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [
    {
     "output_type": "execute_result",
     "execution_count": 0,
     "metadata": {},
     "data": {
      "text/plain": [
       "ResNet(\n",
       "  (conv1): Conv2d(3, 64, kernel_size=(7, 7), stride=(2, 2), padding=(3, 3), bias=False)\n",
       "  (bn1): BatchNorm2d(64, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "  (relu): ReLU(inplace=True)\n",
       "  (maxpool): MaxPool2d(kernel_size=3, stride=2, padding=1, dilation=1, ceil_mode=False)\n",
       "  (layer1): Sequential(\n",
       "    (0): BasicBlock(\n",
       "      (conv1): Conv2d(64, 64, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(64, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(64, 64, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(64, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "    )\n",
       "    (1): BasicBlock(\n",
       "      (conv1): Conv2d(64, 64, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(64, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(64, 64, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(64, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "    )\n",
       "  )\n",
       "  (layer2): Sequential(\n",
       "    (0): BasicBlock(\n",
       "      (conv1): Conv2d(64, 128, kernel_size=(3, 3), stride=(2, 2), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(128, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(128, 128, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(128, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (downsample): Sequential(\n",
       "        (0): Conv2d(64, 128, kernel_size=(1, 1), stride=(2, 2), bias=False)\n",
       "        (1): BatchNorm2d(128, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      )\n",
       "    )\n",
       "    (1): BasicBlock(\n",
       "      (conv1): Conv2d(128, 128, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(128, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(128, 128, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(128, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "    )\n",
       "  )\n",
       "  (layer3): Sequential(\n",
       "    (0): BasicBlock(\n",
       "      (conv1): Conv2d(128, 256, kernel_size=(3, 3), stride=(2, 2), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(256, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(256, 256, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(256, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (downsample): Sequential(\n",
       "        (0): Conv2d(128, 256, kernel_size=(1, 1), stride=(2, 2), bias=False)\n",
       "        (1): BatchNorm2d(256, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      )\n",
       "    )\n",
       "    (1): BasicBlock(\n",
       "      (conv1): Conv2d(256, 256, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(256, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(256, 256, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(256, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "    )\n",
       "  )\n",
       "  (layer4): Sequential(\n",
       "    (0): BasicBlock(\n",
       "      (conv1): Conv2d(256, 512, kernel_size=(3, 3), stride=(2, 2), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(512, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(512, 512, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(512, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (downsample): Sequential(\n",
       "        (0): Conv2d(256, 512, kernel_size=(1, 1), stride=(2, 2), bias=False)\n",
       "        (1): BatchNorm2d(512, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      )\n",
       "    )\n",
       "    (1): BasicBlock(\n",
       "      (conv1): Conv2d(512, 512, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn1): BatchNorm2d(512, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "      (relu): ReLU(inplace=True)\n",
       "      (conv2): Conv2d(512, 512, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)\n",
       "      (bn2): BatchNorm2d(512, eps=1e-05, momentum=0.1, affine=True, track_running_stats=True)\n",
       "    )\n",
       "  )\n",
       "  (avgpool): AdaptiveAvgPool2d(output_size=(1, 1))\n",
       "  (fc): Linear(in_features=512, out_features=1, bias=True)\n",
       ")"
      ]
     }
    }
   ],
   "source": [
    "optimizer = optim.SGD(model.fc.parameters(), lr=0.01)\n",
    "\n",
    "criterion = nn.CrossEntropyLoss()\n",
    "device = torch.device(\"cuda:0\" if torch.cuda.is_available() else \"cpu\")\n",
    "model.to(device)"
   ],
   "id": "19c56ad7-6acf-46fa-80c1-d40133c8e213"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [
    {
     "output_type": "stream",
     "name": "stdout",
     "text": [
      "Train Epoch: 1 [0/244 (0%)] Loss: 9.530968\n",
      "Train Epoch duration 150.19s\n",
      "\n",
      "Test set: Average loss: 2.7408, Accuracy: 0/153 (0%)\n",
      "\n",
      "Train Epoch: 2 [0/244 (0%)] Loss: 9.099474"
     ]
    }
   ],
   "source": [
    "for epoch in range(1, 10):\n",
    "        start_time = time.process_time()\n",
    "        train(model, device, dataloaders[\"train\"], optimizer, epoch, criterion)\n",
    "        end_time = time.process_time()\n",
    "        print(\"Train Epoch duration {:.2f}s\".format(end_time - start_time))\n",
    "        \n",
    "        test(model, device, dataloaders[\"val\"], criterion)"
   ],
   "id": "5adddc99-0071-4983-8e81-048d79f833d8"
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## VGG\n",
    "\n",
    "Voir\n",
    "<https://github.com/pytorch/vision/blob/master/torchvision/models/vgg.py>\n",
    "et\n",
    "<https://pytorch.org/docs/stable/_modules/torch/nn/modules/container.html#Sequential>"
   ],
   "id": "06a1119c-8518-4bec-a326-32bbab92fed7"
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [],
   "id": "93f7f08e-a60e-42d9-ac22-309ad49f9799"
  }
 ],
 "nbformat": 4,
 "nbformat_minor": 5,
 "metadata": {}
}
